/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry.vlib.ejb.impl;

import java.rmi.RemoteException;
import java.sql.Timestamp;
import java.util.Map;

import javax.ejb.CreateException;

/**
 *  Implementation of the Person entity.
 *
 *  <p>We're using container managed persistance.
 *
 *  @see org.apache.tapestry.vlib.ejb.IPerson
 *  @see org.apache.tapestry.vlib.ejb.IPersonHome
 *
 *  @version $Id: PersonBean.java,v 1.2 2003/04/02 19:22:37 hlship Exp $
 *  @author Howard Lewis Ship
 *
 **/

public abstract class PersonBean extends AbstractEntityBean
{

	protected String[] getAttributePropertyNames()
	{
		return new String[] {
			"firstName",
			"lastName",
			"email",
			"password",
			"lockedOut",
			"admin",
			"lastAccess" };
	}

    public abstract void setPersonId(Integer value);
    
    public abstract Integer getPersonId();

	public abstract void setEmail(String value);

	public abstract String getEmail();

	public abstract void setFirstName(String value);

	public abstract String getFirstName();

	public abstract void setLastName(String value);

	public abstract String getLastName();

	public abstract void setPassword(String value);

	public abstract String getPassword();

	public abstract void setLockedOut(boolean value);

	public abstract boolean getLockedOut();

	public abstract void setAdmin(boolean value);

	public abstract boolean getAdmin();

	public abstract void setLastAccess(Timestamp value);

	public abstract Timestamp getLastAccess();

	public Integer ejbCreate(Map attributes) throws CreateException, RemoteException
	{
        updateEntityAttributes(attributes);
        
        setPersonId(allocateKey());
        
        return null;
	}

	public void ejbPostCreate(Map attributes)
	{
		// Do nothing
	}
}