/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 *  Implementation of the {@link org.apache.tapestry.ILocation} interface.
 *
 *  @author Howard Lewis Ship
 *  @version $Id: Location.java,v 1.4 2003/06/19 10:58:52 hlship Exp $
 *  @since 3.0
 *
 **/
public class Location implements ILocation
{
    private IResourceLocation _resourceLocation;
    private int _lineNumber = -1;
    private int _columnNumber = -1;

    public Location(IResourceLocation location)
    {
        _resourceLocation = location;
    }

    public Location(IResourceLocation location, int lineNumber)
    {
        this(location);

        _lineNumber = lineNumber;
    }

    public Location(IResourceLocation location, int lineNumber, int columnNumber)
    {
        this(location);

        _lineNumber = lineNumber;
        _columnNumber = columnNumber;
    }

    public IResourceLocation getResourceLocation()
    {
        return _resourceLocation;
    }

    public int getLineNumber()
    {
        return _lineNumber;
    }

    public int getColumnNumber()
    {
        return _columnNumber;
    }

    public int hashCode()
    {
        HashCodeBuilder builder = new HashCodeBuilder(237, 53);

        builder.append(_resourceLocation);
        builder.append(_lineNumber);
        builder.append(_columnNumber);

        return builder.toHashCode();
    }

    public boolean equals(Object other)
    {
        if (!(other instanceof ILocation))
            return false;

        ILocation l = (ILocation) other;

        EqualsBuilder builder = new EqualsBuilder();
        builder.append(_lineNumber, l.getLineNumber());
        builder.append(_columnNumber, l.getColumnNumber());
        builder.append(_resourceLocation, l.getResourceLocation());

        return builder.isEquals();
    }

    public String toString()
    {
        if (_lineNumber <= 0 && _columnNumber <= 0)
            return _resourceLocation.toString();
        StringBuffer buffer = new StringBuffer(_resourceLocation.toString());
        if (_lineNumber > 0)
        {
            buffer.append(", line ");
            buffer.append(_lineNumber);
        }

        if (_columnNumber > 0)
        {
            buffer.append(", column ");
            buffer.append(_columnNumber);
        }

        return buffer.toString();
    }
}
