/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry.engine;

import org.apache.tapestry.spec.IComponentSpecification;

/**
 *
 *  A provider of enhanced classes, classes with new methods 
 *  and new attributes, and possibly, implementing new
 *  Java interfaces.  The primary use of class enhancement is to
 *  automate the creation of transient and persistant properties.
 * 
 *  <p>
 *  Implementations of this interface must be threadsafe.
 *
 *  @author Howard Lewis Ship
 *  @version $Id: IComponentClassEnhancer.java,v 1.3 2003/04/17 21:33:51 hlship Exp $
 *  @since 3.0
 * 
 **/

public interface IComponentClassEnhancer
{
	/**
	 *  Clears all cached data for the enhancer; this includes references to
	 *  enhanced classes.
	 * 
	 **/
	
	public void reset();
	
	/**
	 *  Used to access the class for a given component (or page).  Returns the
	 *  specified class, or an enhanced version of the class if the
	 *  component requires enhancement.
	 * 
	 *  @param specification the specification for the component
	 *  @param className the name of base class to enhance, as extracted
	 *  from the specification (or possibly, from a default).
	 * 
	 *  @throws ApplicationRuntimeException if the class does not exist, is invalid,
	 *  or may not be enhanced.
	 * 
	 **/
	
	public Class getEnhancedClass(IComponentSpecification specification, String className);
}
