/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry.multipart;

import java.util.ArrayList;
import java.util.List;

/**
 *  A portion of a multipart request that stores a value, or values, for
 *  a parameter.
 *
 *  @author Howard Lewis Ship
 *  @version $Id: ValuePart.java,v 1.2 2003/03/23 16:12:38 hlship Exp $
 *  @since 2.0.1
 *
 **/

public class ValuePart implements IPart
{
    private int _count;
    // Stores either String or List of String
    private Object _value;

    public ValuePart(String value)
    {
        _count = 1;
        _value = value;
    }

    public int getCount()
    {
        return _count;
    }

    /**
     *  Returns the value, or the first value (if multi-valued).
     * 
     **/

    public String getValue()
    {
        if (_count == 1)
            return (String) _value;

        List l = (List) _value;

        return (String) l.get(0);
    }

    /**
     *  Returns the values as an array of strings.  If there is only one value,
     *  it is returned wrapped as a single element array.
     * 
     **/

    public String[] getValues()
    {
        if (_count == 1)
            return new String[] {(String) _value };

        List l = (List) _value;

        return (String[]) l.toArray(new String[_count]);
    }

    public void add(String newValue)
    {
        if (_count == 1)
        {
            List l = new ArrayList();
            l.add(_value);
            l.add(newValue);

            _value = l;
            _count++;
            return;
        }

        List l = (List) _value;
        l.add(newValue);
        _count++;
    }

    /**
     *  Does nothing.
     * 
     **/

    public void cleanup()
    {
    }
}