/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation", "Tapestry" 
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache" 
 *    or "Tapestry", nor may "Apache" or "Tapestry" appear in their 
 *    name, without prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE TAPESTRY CONTRIBUTOR COMMUNITY
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.tapestry.util;

/**
 *  Used to split a string into substrings based on a single character
 *  delimiter.  A fast, simple version of
 *  {@link java.util.StringTokenizer}.
 *
 *  @author Howard Lewis Ship
 *  @version $Id: StringSplitter.java,v 1.1 2003/03/05 22:59:33 hlship Exp $
 * 
 **/

public class StringSplitter
{
    private char delimiter;

    public StringSplitter(char delimiter)
    {
        this.delimiter = delimiter;
    }

    public char getDelimiter()
    {
        return delimiter;
    }

    /**
     *  Splits a string on the delimter into an array of String
     *  tokens.  The delimiters are not included in the tokens.  Null
     *  tokens (caused by two consecutive delimiter) are reduced to an
     *  empty string. Leading delimiters are ignored.
     *
     **/

    public String[] splitToArray(String value)
    {
        char[] buffer;
        int i;
        String[] result;
        int resultCount = 0;
        int start;
        int length;
        String token;
        String[] newResult;
        boolean first = true;

        buffer = value.toCharArray();

        result = new String[3];

        start = 0;
        length = 0;

        for (i = 0; i < buffer.length; i++)
        {
            if (buffer[i] != delimiter)
            {
                length++;
                continue;
            }

            // This is used to ignore leading delimiter(s).

            if (length > 0 || !first)
            {
                token = new String(buffer, start, length);

                if (resultCount == result.length)
                {
                    newResult = new String[result.length * 2];

                    System.arraycopy(result, 0, newResult, 0, result.length);

                    result = newResult;
                }

                result[resultCount++] = token;

                first = false;
            }

            start = i + 1;
            length = 0;
        }

        // Special case:  if the string contains no delimiters
        // then it isn't really split.  Wrap the input string
        // in an array and return.  This is a little optimization
        // to prevent a new String instance from being
        // created unnecessarily.

        if (start == 0 && length == buffer.length)
        {
            result = new String[1];
            result[0] = value;
            return result;
        }

        // If the string is all delimiters, then this
        // will result in a single empty token.

        token = new String(buffer, start, length);

        newResult = new String[resultCount + 1];
        System.arraycopy(result, 0, newResult, 0, resultCount);
        newResult[resultCount] = token;

        return newResult;
    }
}